@@ -47,9 +47,10 @@ use Cwd qw(abs_path);
 use Carp;
 use Data::Dumper;
 use File::Basename qw(basename);
+use Encode qw(encode decode);
 
 # see http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-our $VERSION = "1.4";
+our $VERSION = "1.5";
 $VERSION = eval $VERSION;
 
 binmode STDOUT, ':utf8';
@@ -98,8 +99,11 @@ sub start_plugin {
 
     # Save the config for this plugin to string,
     # we will feed it to the plugin via stdin below.
-    my $config     = $conf->obj('monitor')->obj($name);
-    my $config_str = $config->save_string();
+    my $config = $conf->obj('monitor')->obj($name);
+    # We need to specifically encode the config string
+    # because AnyEvent does not
+    # specify an encoding.
+    my $config_str = encode('UTF-8', $config->save_string());
 
     my @dest;
     if ($config->exists('send_alerts_to')) {
@@ -141,7 +145,7 @@ sub start_plugin {
     @start_request = (
         json => sub {
             my ($hdl, $hashref) = @_;
-            handle_stderr_msg(basename($plugin), \@dest, $hashref);
+            handle_stderr_msg($name, basename($plugin), \@dest, $hashref);
             $hdl->push_read(@start_request);
         });
 
@@ -165,11 +169,56 @@ sub start_plugin {
         });
 }
 
+sub message_silenced {
+    my ($message, $counts) = @_;
+
+    my $pattern = $message->{silenced_by};
+    if (!defined($pattern) &&
+        $conf->exists('silenced_by')) {
+        $pattern = $conf->value('silenced_by');
+    }
+
+    return unless defined($pattern);
+
+    my @matches;
+    my @ids = grep { $_ ne $message->{failure_id} } keys %$counts;
+    if (my ($re) = ($pattern =~ m,^/(.*)/$,)) {
+        # Regular expression matching
+        my $compiled = qr/$re/;
+        @matches = grep { /$compiled/ } @ids;
+    } else {
+        # String prefix matching
+        my $prefix = quotemeta($pattern);
+        @matches = grep { m,^$prefix, } @ids;
+    }
+
+    return @matches > 0;
+}
+
 sub xmpp_empty_queue {
     my @leftover;
 
+    my $consecutive_failures = (
+        $conf->exists('consecutive_failures') ?
+            $conf->value('consecutive_failures') :
+            0
+    );
+
+    my %counts;
     for my $entry (@queued_messages) {
         my ($jid, $message) = @$entry;
+        $counts{ $message->{failure_id} }++;
+    }
+
+    for my $entry (@queued_messages) {
+        my ($jid, $message) = @$entry;
+
+        next if message_silenced($message, \%counts);
+
+        if ($counts{ $message->{failure_id} } < $consecutive_failures) {
+            push @leftover, $entry unless $message->{expiration} < time();
+            next;
+        }
 
         # Find the account,
         # if unsuccessful,
@@ -195,15 +244,14 @@ sub xmpp_empty_queue {
             $presence->jid,
             'chat',
             undef,
-            body => $message
-        );
+            body => $message->{body});
     }
 
     @queued_messages = @leftover;
 }
 
 sub handle_stderr_msg {
-    my ($module, $dest, $data) = @_;
+    my ($name, $module, $dest, $data) = @_;
     if (!exists($data->{severity}) ||
         !exists($data->{message})) {
         say STDERR
@@ -211,10 +259,27 @@ sub handle_stderr_msg {
         return;
     }
 
+    my $module_config = $conf->obj('monitor')->obj($name);
+    my $interval      = 60;
+    if ($module_config->exists('interval')) {
+        $interval = $module_config->value('interval');
+    }
+    my $silenced_by = undef;
+    if ($module_config->exists('silenced_by')) {
+        $silenced_by = $module_config->value('silenced_by');
+    }
+
     if ($data->{severity} eq 'critical') {
-        say "relaying: " . $data->{message};
+        say "read from plugin: " . $data->{message};
+        my $message = {
+            body        => $data->{message},
+            expiration  => time() + $interval,
+            failure_id  => $data->{id} // $data->{message},
+            silenced_by => $silenced_by,
+        };
+
         for my $jid (@$dest) {
-            push @queued_messages, [ $jid, $data->{message} ];
+            push @queued_messages, [ $jid, $message ];
         }
         xmpp_empty_queue();
     }
@@ -533,7 +598,7 @@ the POD documentation is intentionally sparse.
 
 =head1 VERSION
 
-Version 1.4
+Version 1.5
 
 =head1 AUTHOR
 
@@ -541,7 +606,7 @@ Michael Stapelberg, C<< <michael at stapelberg.de> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012-2013 Michael Stapelberg.
+Copyright 2012-2014 Michael Stapelberg.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the BSD license.
