@@ -34,7 +34,7 @@ our @EXPORT = qw(
 our $conf;
 
 # see http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-our $VERSION = "1.4";
+our $VERSION = "1.5";
 $VERSION = eval $VERSION;
 
 # This will be set to a true value
@@ -49,11 +49,17 @@ my $errorfh;
 my $main_timer;
 
 sub signal_error {
-    my ($severity, $message) = @_;
-    say $errorfh encode_json({
-            severity => $severity,
-            message  => $message
-    });
+    my ($severity, $message, $id) = @_;
+    my $json = {
+        severity => $severity,
+        message  => $message
+    };
+
+    if (defined($id) && ref $id eq 'ARRAY') {
+        $json->{id} = join('.', @$id);
+    }
+
+    say $errorfh encode_json($json);
 }
 
 sub import {
@@ -118,12 +124,17 @@ EOT
             }
         } until ($conf->exists('family'));
 
-        # TODO: parse interval from config
-        my $interval = 60;
+        my $interval =
+            ($conf->exists('interval') ? $conf->value('interval') : 60);
+
+        # Ensure timeout is an int and > 0.
+        $interval += 0;
+        $interval ||= 1;
 
         # Periodically run the check, but don’t wait for the first $interval seconds to
         # pass, but run it right now, too.
-        my $run; $run = sub {
+        my $run;
+        $run = sub {
             # XXX ->can also resolves in @ISA
             die 'plugins must implement "sub run"'
                 unless $run = $pkg->can("run");
